//-------------------------------------------------------------------------------------
// PORTS.CPP
//
// Opens, closes, and sets values on ports of the ADS ThinClient
//
// History
// Written by J. Baik 17Mar99
// 21Apr99 ak
//   - change Open/ClosePort to Open/CloseLEDPort, delete dialog references,
//		and add the SetLEDPort() function
// 26Apr99
//   - remove "LED" from function names to make general
//   - rename from LED.H to PORTS.H
// 16Dec99 ak
//   - add functions Port_is_Valid(), File_exists(), Read_Port(), 
//		Port_is_High(), and Set_Port()
// 17Dec99 ak
//   - add showError flag on Open_Port(), Port_is_Valid() and File_exists()
// 01Apr00 ak
//   - include Stdafx.h and Ports.h; modify Ports.h to be included only once
//	 - add the Read_Port_Word() function
//	 - rename Read_Port() to Read_Port_Byte()
//   - rename Set_Port() to Set_Port_Byte()
// 18Apr00 ak
//   - don't use StdAFX.h so it's more portable
// 5feb02 ak
//   - without stdafx.h I get "unexpected end of file looking for precompiled header"
//-------------------------------------------------------------------------------------

#include "stdafx.h"
#define WIN32_LEAN_AND_MEAN	// Exclude rarely-used stuff from windows include files
#include <windows.h>

#include "ports.h"

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
BOOL Open_Port(HANDLE &hPort, LPWSTR port, BOOL showError)
{
	hPort = CreateFile (port, 				// Port Name (Unicode compatible)
			GENERIC_READ | GENERIC_WRITE, 	// Open for Read-Write
            0,            					// COM port cannot be shared
            NULL,          					// Always NULL for Windows CE
            OPEN_EXISTING, 					// allows R, W and R/W operation
            0,             					// Non-overlapped operation only
            NULL);         					// Always NULL for Windows CE

	if(hPort == INVALID_HANDLE_VALUE)
	{
		if (showError)
			DEBUGMSG (1,(L"   Open_Port() unable to open '%s'\r\n",port));
		return FALSE;
	}
	else
	{
		return TRUE;
	}
}

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
BOOL Close_Port(HANDLE &hPort)
{
	BYTE theByte = 0;

	if(hPort != INVALID_HANDLE_VALUE)
	{
		CloseHandle(hPort);
		hPort = INVALID_HANDLE_VALUE;
	}
	return TRUE;
}

//-------------------------------------------------------------------------------------
//  Tests to make sure a port can be opened.  Good way to find out if port
//  is valid and available, especially the first time you use the port.
//
//   port		port name (e.g. "LED1:", "UOA3", etc)
//-------------------------------------------------------------------------------------
BOOL Port_is_Valid(LPWSTR port, BOOL showError)
{
	HANDLE hPort;

    if (Open_Port(hPort, port, showError)) 
	{
		Close_Port(hPort);
		return TRUE;
	}
	else
		return FALSE;
}

//-------------------------------------------------------------------------------------
// Tests if a file is valid (same as testing for a valid port)
//-------------------------------------------------------------------------------------
BOOL File_exists(LPWSTR port, BOOL showError)
{
	return Port_is_Valid(port, showError);
}


//-------------------------------------------------------------------------------------
// Reads a BYTE (8-bit) from the CE port.
//
// Error codes are:
//		-1	can't open port
//		-2	invalid number of bytes returned
//-------------------------------------------------------------------------------------
int Read_Port_Byte (LPWSTR port)
{
	HANDLE hPort;
	DWORD dwBytesRead;
	BYTE ReadByte;

    if (Open_Port(hPort, port, FALSE)) {

		ReadFile(hPort,					// where to read from (the open port)
					&ReadByte,			// byte to read
					1,					// number of bytes to be read from port
					&dwBytesRead,		// number of bytes that were actually read
					NULL);				// overlapped I/O not supported by CE
		Close_Port(hPort);
		if (dwBytesRead != 1)
			return -2;
		else
			return ReadByte;
	}
	else
		return -1;
}

//-------------------------------------------------------------------------------------
// Reads a WORD (16-bit) from the CE port.
//
// Error codes are:
//		-1	can't open port
//		-2	invalid number of bytes returned
//-------------------------------------------------------------------------------------
int Read_Port_Word (LPWSTR port)
{
	HANDLE hPort;
	DWORD dwBytesRead;
	WORD ReadWord;

    if (Open_Port(hPort, port, FALSE)) {

		ReadFile(	hPort,				// where to read from (the open port)
					&ReadWord,			// data read
					2,					// number of bytes to be read from port
					&dwBytesRead,		// number of bytes that were actually read
					NULL);				// overlapped I/O not supported by CE
		Close_Port(hPort);
		if (dwBytesRead != 2)
			return -2;
		else
			return ReadWord;
	}
	else
		return -1;
}

//-------------------------------------------------------------------------------------
//  Read a value from a port.  Returns TRUE if port is set to logical 1.
//
//  Note that if the port does not open correctly, FALSE is always returned.
//  You should test a port first with Port_is_Valid().
//
//   port		port name (e.g. "LED1:", "UOA3", etc)
//-------------------------------------------------------------------------------------
BOOL Port_is_High (LPWSTR port)
{
	int j;

	j = Read_Port_Byte(port);
	if (j < 0)
		return FALSE;
	else
		return (j==1);

}



//-------------------------------------------------------------------------------------
//  Sends a value to a port.  Opens and closes port.  Good for driving LEDs 
//  and IO ports.
//
//   port		port name (e.g. "LED1:", "UOA3", etc)
//   setting	value appropriate to port; for LEDs and IOs, this is generally
//				0 for off and 1 for on
//-------------------------------------------------------------------------------------
BOOL Set_Port_Byte (LPWSTR port, BYTE setting)
{
	HANDLE hPort;
	DWORD dwBytesWritten;
	BYTE byteToWrite = setting;

// RETAILMSG(1,(L" %s%d  ",port, byteToWrite));  // debug

    if (Open_Port(hPort, port, FALSE)) {

		WriteFile(hPort,				// where to write to (the open port)
					&byteToWrite,		// what to write
					1,					// number of bytes to be written to port
					&dwBytesWritten,	// number of bytes that were actually written
					NULL);				// overlapped I/O not supported			
		Close_Port(hPort);
		return TRUE;
	}
	else
		return FALSE;
}
